from wsknn.fit_transform import fit
from wsknn.predict import predict
from wsknn.model.wsknn import WSKNN
from wsknn.preprocessing.parse_static import parse_files
from wsknn.preprocessing.structure.item import Items
from wsknn.preprocessing.structure.session import Sessions


__version__ = '1.1.1'
